/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

#include <stdio.h>
#include <string.h>
#include "conf.h"

char *emptyMapping = "";

ConfigurationFile::ConfigurationFile ()
{
    m_pFile = NULL;
}

ConfigurationFile::ConfigurationFile (PSTR pszFileName)
{
    if (m_pFile) {
	cerr << "ConfigurationFile has alread been opened" << endl;
	return;
    }
    if (m_pFile = fopen (pszFileName, "r"))
	m_pszFileName = strdup (pszFileName);
}

ConfigurationFile::~ConfigurationFile ()
{
    fclose (m_pFile);
}

int
ConfigurationFile::open (PSTR pszFileName)
{
    if (m_pFile) {
	cerr << "ConfigurationFile has been opened already" << endl;
	return 0;
    }
    if (m_pFile = fopen (pszFileName, "r")) {
	m_pszFileName = strdup (pszFileName);
	return 0;
    } else {
	return 1;
    }
}

void
ConfigurationFile::close (void)
{
    if (m_pFile) {
	cerr << "ConfigurationFile had not been opened before closing." << endl;
	return;
    }
    fclose (m_pFile);
    m_pFile = NULL;
}

Pair *
ConfigurationFile::nextEntry (void)
{
    char	buffer[1024], *p;
    int		nCount;

    do {
	nCount = readLine (buffer, sizeof(buffer)-1);
	p = buffer;
	while (*p == ' ' || *p == '\t' || *p == '\n')
	    p++;
    } while (nCount && (buffer[0] == '#' || *p == '\0'));

    if (!nCount)
	return NULL;

    // try to find the delimiter. Either space or tab
    p = buffer;
    while (*p != ' ' && *p != '\t' && *p != '\0')
	p++;

    Pair *pPair = new Pair;
    if (*p == '\n' || *p == '\0') {
	// could not find any value, make value to point to empty string
	pPair->key = strdup (buffer);
	pPair->value = emptyMapping;
    } else {
	// found delimiter, copy rest of the line to value field
	*p = '\0'; p++;
	while (*p == ' ' || *p == '\t')
	    p++;
	pPair->key = strdup (buffer);
	pPair->value = strdup (p);
    }
    return (pPair);
}

int
ConfigurationFile::readLine (PSTR pBuffer, long lCount)
{
    if (!m_pFile) {
	cerr << "Trying to read from a configuration file which is not yet opened." << endl;
	exit (1);
    }
    int len = fgets (pBuffer, lCount, m_pFile) ? strlen (pBuffer) : 0;
    if (len) pBuffer[len-1] = '\0';

    return len;
}

