%{
/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

static char *rcsid = "$Revision$";

#include <ctype.h>
#include "node.h"
#include "maths.h"
#include "list.h"
#include "y.tab.h"

extern char	*pSource;

int inTableLevel = 0;
int inTableSpecs = 0;
int inOptionalLevel = 0;

void removeNextSpaces (void);
int nextIsBracket (void);

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)			\
{							\
    if (*pSource == 0) {				\
	    buf[0] = '\0';				\
	    result = YY_NULL;				\
    } else {						\
	if (strlen(pSource) <= max_size) {		\
	    strcpy (buf, pSource);			\
	    result = strlen(buf);			\
	    pSource += result*sizeof(char);		\
        } else {					\
	    strncpy (buf, pSource, max_size);		\
	    buf[max_size] = '\0';			\
	    result = max_size;				\
	    pSource += max_size*sizeof(char);		\
	}						\
    }							\
}

%}

%s optional intable
%x intablespecs

%%

[ ]+				{ yylval.s = strdup(" "); return PLAIN; }
[\t\n]				{ ; /* Pass them. */ }
<INITIAL,intable,intablespecs>\%.*\n	{ ; /* Pass comments */ }
\_				{ /* is there one literal character or a start
				     of a new group?
				     Add the braces if literal */
    				  int c; char buf[256]; int bufcounter = 0;
				  /* skip extra white space */
				  removeNextSpaces();
				  c = yyinput ();
				  if (c == '\\') {
				      do {
					  buf[bufcounter++] = c;
					  c = yyinput();
				      } while (isalpha (c));
				      unput (c);
				      unput ('}');
				      while (--bufcounter >= 0)
					  unput (buf[bufcounter]);
				      unput ('{');
				  } else {
				      if (c != '{') {
					  unput ('}');
					  unput (c);
					  unput ('{');
				      } else {
					  unput (c);
				      }
				  }
				  return BEGIN_SUBSCRIPT; }
\^				{ /* is there one literal character or a start
				     of a new group?
				     Add the braces if literal */
    				  int c; char buf[256]; int bufcounter = 0;
				  /* skip extra white space */
				  removeNextSpaces();
				  c = yyinput();
				  if (c == '\\') {
				      do {
					  buf[bufcounter++] = c;
					  c = yyinput();
				      } while (isalpha (c));
				      unput (c);
				      unput ('}');
				      while (--bufcounter >= 0)
					  unput (buf[bufcounter]);
				      unput ('{');
				  } else {
				      if (c != '{') {
					  unput ('}');
					  unput (c);
					  unput ('{');
				      } else {
					  unput (c);
				      }
				  }
				  return BEGIN_SUPERSCRIPT; }
<INITIAL,intable>\{		{ if (inTableSpecs)
				      inTableSpecs++;
				  return BEGIN_GROUP; }
<INITIAL,intable>\}		{ if (inTableSpecs) {
    				      if (--inTableSpecs == 1)
					  BEGIN(intablespecs);
				  }
				  removeNextSpaces();
				  return END_GROUP; }
<INITIAL,intable>\]		{ if (inTableSpecs) {
    				      --inTableSpecs;
				      BEGIN(intablespecs);
				      removeNextSpaces();
				      return END_ARG;
				  } else {
				      yylval.s = strdup (yytext);
				      removeNextSpaces();
				      return PLAIN;
				  } }
<intablespecs>\{		{ if (++inTableSpecs > 1)
				      BEGIN(intable);
				  return BEGIN_GROUP; }
<intablespecs>\}		{ if (!--inTableSpecs)
				      BEGIN(intable);
				  removeNextSpaces();
				  return END_GROUP; }
<intablespecs>\[		{ ++inTableSpecs;
				  BEGIN(intable);
				  removeNextSpaces();
				  return BEGIN_ARG; }
<intablespecs>\]		{ if (!--inTableSpecs)
				      BEGIN(intable);
				  removeNextSpaces();
				  return END_ARG; }
<intablespecs>>			{ return BEGIN_PREDECL; }
<intablespecs><			{ return BEGIN_POSTDECL; }
<intablespecs>@			{ yylval.c = *yytext; return AT_WIDTH; }
<intablespecs>m			{ yylval.c = *yytext; return M_WIDTH; }
<intablespecs>b			{ yylval.c = *yytext; return B_WIDTH; }
<intablespecs>p			{ yylval.c = *yytext; return P_WIDTH; }
<intablespecs>l|L		{ yylval.c = *yytext; return LEFT_ALIGN; }
<intablespecs>c|C		{ yylval.c = *yytext; return CENTER_ALIGN; }
<intablespecs>r|R		{ yylval.c = *yytext; return RIGHT_ALIGN; }
<intablespecs>\|		{ return BAR; }
<intablespecs>!			{ return EXCLAMATION; }
<intablespecs>\*		{ char valbuf[10];
				  int x, y, value, count = 0;
				  int level = 0;
				  int c = yyinput();
				  if (c == '{') {
					do {
					    c = yyinput();
					    valbuf[count++] = c;
				      } while (c != '}');
				      valbuf[count] = '\0';
				      value = atoi(valbuf);
				  } else {
				      value = c - '0';
				  }
				  count = 0;
				  for (;;) {
				      c = yyinput ();
				      if (c == '{') {
					  if (++level > 1)
					      valbuf[count++] = c;
				      } else if (c == '}') {
					  if (!--level)
					      break;
					  if (level > 0)
					      valbuf[count++] = c;
				      } else
					  valbuf[count++] = c;
					  
				  }
				  valbuf[count] = '\0';
				  
				  for (x = 0; x < value; x++)
				      for (y = count-1; y >= 0; y--) {
					  unput (valbuf[y]);
				      } }
\\sum				{ return SUM; }
\\fraction			{ removeNextSpaces(); return FRACTION; }
\\frac				{ removeNextSpaces(); return FRACTION; }
\\int				{ removeNextSpaces(); return INTEGRAL; }
\\sqrt				{ removeNextSpaces();
    				  if (nextIsBracket()) {
				      inOptionalLevel++;
				      BEGIN(optional);
				  }
				  return ROOT; }
<optional>\{			{ BEGIN(INITIAL);
				  if (inTableLevel)
				      BEGIN(intable);
				  return BEGIN_GROUP; }
<optional>\[			{ return BEGIN_ARG; }
<optional>\]			{ if (!--inOptionalLevel)
				      BEGIN(INITIAL);
				  if (inTableLevel)
				      BEGIN(intable);
				  if (nextIsBracket()) {
				      inOptionalLevel++;
				      BEGIN(optional);
				  }
				  return END_ARG; }
\\\[				{ removeNextSpaces(); return MATHEMATICS; }
\\\]				{ removeNextSpaces(); return MATHEMATICS; }
\$[\$]?				{ removeNextSpaces(); return MATHEMATICS; }
\\begin\{eqnarray\}		{ inTableLevel++;
				  BEGIN(intable);
				  return BEGIN_EQNARRAY; }
<intable>\\end\{eqnarray\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_EQNARRAY; }
\\begin\{eqnarray\*\}		{ inTableLevel++;
				  BEGIN(intable);
				  return BEGIN_EQNARRAY; }
<intable>\\end\{eqnarray\*\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_EQNARRAY; }
\\begin\{eqnarray\*\*\}		{ inTableLevel++;
				  BEGIN(intable);
				  return BEGIN_EQNARRAY; }
<intable>\\end\{eqnarray\*\*\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_EQNARRAY; }
\\begin\{array\}		{ inTableLevel++;
				  BEGIN(intablespecs);
				  return BEGIN_ARRAY; }
<intable>\\end\{array\}		{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_ARRAY; }
\\begin\{tabular\}		{ inTableLevel++;
				  BEGIN(intablespecs);
				  return BEGIN_TABULAR; }
<intable>\\end\{tabular\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_TABULAR; }
\\begin\{tabular\*\}		{ if (nextIsBracket()) {
    				    inTableLevel++;
				    BEGIN(intablespecs);
				  }
				  return BEGIN_TABULAR_STAR; }
<intable>\\end\{tabular\*\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_TABULAR_STAR; }
\\begin\{longtable\}		{ inTableLevel++;
				  BEGIN(intablespecs);
				  return BEGIN_LONGTABLE; }
<intable>\\end\{longtable\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_LONGTABLE; }
\\begin\{tabbing\}		{ inTableLevel++;
				  BEGIN(intablespecs);
				  return BEGIN_TABBING; }
<intable>\\end\{tabbing\}	{ inTableLevel = 0;
				  BEGIN(INITIAL);
				  removeNextSpaces();
				  return END_TABBING; }
<intable>\\\\[ ]*\[[^\]]+\]	{ return END_OF_ROW; }
<intable>\\\\			{ removeNextSpaces();
				  return END_OF_ROW; }
<intable>&			{ removeNextSpaces();
				  return COLUMN_SEPARATOR; }
\\[,:;!_/\{\}]			{ yylval.s = strdup(yytext);
				  removeNextSpaces();
				  return SEQUENCE; }
\\left				{ return LEFT; }
\\right				{ return RIGHT; }
\\[bB]ig[g]?[rl]		{ char buf[2];
				  buf[1] = '\0';
				  buf[0] = yyinput();
				  yylval.s = strdup (buf);
				  return PLAIN; }
\\[a-zA-Z%_][a-zA-Z]*[\*]*	{ yylval.s = strdup(yytext);
				  if (nextIsBracket()) {
				      inOptionalLevel++;
				      BEGIN(optional);
				  }
				  removeNextSpaces();
				  return SEQUENCE; }
\\				{ ; /* pass single backslashes */ }
<optional>[^ \t\n&$^{}_\\\]\[]+	{ yylval.s = strdup(yytext); return PLAIN; }
<INITIAL,intable>[^ \t\n&$^{}_\\\]]+ { yylval.s = strdup(yytext); return PLAIN; }
.				{ return *yytext; }



%%

void removeNextSpaces (void)
{
    int ch;

    do {
	ch = yyinput();
    } while (ch == ' ' || ch == '\t' || ch == '\n');
    if (ch != -1)
	unput (ch);
}

int nextIsBracket (void)
{
    int retval = 0;
    int c = yyinput();

    /* bracket has to be the next character without any spaces */
    if (c == '[')
	retval = 1;
    unput (c);
    return retval;
}
