/***************************************************************************\
 *                                     Tab2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

static char *rcsid = "$Revision$";

extern "C" {
#include "stdlib.h"
};

#include "list.h"

#define FOR(c, l, b) \
  do { \
    if (!(l).empty()) { \
      ListCursor c(l); \
      do { \
        b \
        c.goNext(); \
      } while (c.valid()); \
    } \
  } while (FALSE)

List::List(List &l)
{
    m_pFirst = m_pLast = NULL;
    FOR(c, l, { addLast(c); });
}


List::~List(void)
{
    if (!empty()) {
	ListCursor c(*this);
	c.goFirst();
	do {
	    Node *p = (Node *) c;
	    if (p) {
		delete p;
	    }
	    c.goNext();
	} while (c.valid());
    }
    while (!empty())
	dropFirst ();
}

void
List::convert_html3 (void)
{
    if (!empty()) {
	ListCursor c(*this);
	c.goFirst();
	do {
	    Node *p = (Node *) c;
	    if (p) {
		p->convert_html3();
	    }
	    c.goNext();
	} while (c.valid());
    }
}

void
List::convert_12083 (void)
{
    if (!empty()) {
	ListCursor c(*this);
	c.goFirst();
	do {
	    Node *p = (Node *) c;
	    if (p) {
		p->convert_12083 ();
	    }
	    c.goNext();
	} while (c.valid());
    }
}

void
List::assign(List &l)
{
    if (this == &l)
	return;
    while (!empty())
	dropFirst();
    FOR(c, l, { addLast(c); });
}


void
List::addFirst(Node *n)
{
    ListNode *list_node = new ListNode (n, NULL, m_pFirst);
    if (empty()) {
	m_pFirst = m_pLast = list_node;
	return;
    }
    m_pFirst->previous = list_node;
    m_pFirst = list_node;
}


void
List::addLast(Node *n)
{
    ListNode *list_node = new ListNode(n, m_pLast, NULL);
    if (empty()) {
	m_pFirst = m_pLast = list_node;
	return;
    }
    m_pLast->next = list_node;
    m_pLast = list_node;
}


void
List::dropFirst(void)
{
    if (empty()) {
	cerr << "Tried to dropFirst() from an empty List.\n";
	exit(1);
    }
    if (singleton()) {
	delete m_pFirst;
	m_pFirst = m_pLast = NULL;
	return;
    }
    ListNode *tmp = m_pFirst;
    m_pFirst = m_pFirst->next;
    m_pFirst->previous = NULL;
    delete tmp;
}

void
List::dropLast(void)
{
    if (empty()) {
	cerr << "Tried to dropLast() from an empty List.\n";
	exit(1);
    }
    if (singleton()) {
	delete m_pLast;
	m_pFirst = m_pLast = NULL;
	return;
    }
    ListNode *tmp = m_pLast;
    m_pLast = m_pLast->previous;
    m_pLast->next = NULL;
    delete tmp;
}

int
List::items(void)
{
    int l = 0;
    FOR(c, *this, { l++; });
    return l;
}

void
ListCursor::addNext(Node *n)
{
    if (!valid()) {
	cerr << "Tried to addNext() to an invalid ListCursor.\n";
	exit(1);
    }
    ListNode *list_node = new ListNode(n, cursor, cursor->next);
    if (!atLast())
	cursor->next->previous = list_node;
    else
	list->m_pLast = list_node;
    cursor->next = list_node;
}


void
ListCursor::addPrevious(Node *n)
{
    if (!valid()) {
	cerr << "Tried to addPrevious() to an invalid ListCursor.\n";
	exit(1);
    }
    ListNode *list_node = new ListNode(n, cursor->previous, cursor);
    if (!atFirst())
	cursor->previous->next = list_node;
    else 
	list->m_pFirst = list_node;
    cursor->previous = list_node;
}


void
ListCursor::dropNext(void)
{
    if (!valid()) {
	cerr << "Tried to dropNext() of an invalid ListCursor.\n";
	exit(1);
    }
    if (atLast()) {
	cerr << "Tried to dropNext() of a ListCursor at its m_pLast.\n";
	exit(1);
    }
    if (cursor->next == list->m_pLast) {
	list->m_pLast = cursor;
	delete cursor->next;
	cursor->next = NULL;
	return;
    }
    ListNode *tmp = cursor->next;
    cursor->next = cursor->next->next;
    if (cursor->next != NULL)
	cursor->next->previous = cursor;
    delete tmp;
}


void
ListCursor::dropPrevious(void)
{
    if (!valid()) {
	cerr << "Tried to dropPrevious() of an invalid ListCursor.\n";
	exit(1);
    }
    if (atFirst()) {
	cerr << "Tried to dropPrevious() of a ListCursor at its m_pFirst.\n";
	exit(1);
    }
    if (cursor->previous == list->m_pFirst) {
	list->m_pFirst = cursor;
	delete cursor->previous;
	cursor->previous = NULL;
	return;
    }
    ListNode *tmp = cursor->previous;
    cursor->previous = cursor->previous->previous;
    if (cursor->previous != NULL)
	cursor->previous->next = cursor;
    delete tmp;
}

ColList::ColList ()
{
    m_pSpecifiers = NULL;
}

ColList::~ColList ()
{
    // do not delete m_pSpecifiers, will be deleted by Table
}

void
ColList::specifiers (List *pSpecifiers)
{
    m_pSpecifiers = pSpecifiers;
}

void
ColList::convert_html3 (void)
{
    if (!m_pSpecifiers)
	return;

    // convert separate cells
    strcat (pResult, "\n<tr>");
    if (!empty() && !m_pSpecifiers->empty()) {
	ListCursor c(*this);
	ListCursor d(*m_pSpecifiers);
	c.goFirst();
	d.goFirst();
	do {
	    Node *p = (Node *) c;
	    ColSpecifier *pCS = (ColSpecifier *)d;
	    if (p) {
		strcat (pResult, "<td align=");
		switch (pCS->m_cAlignment) {
		case 'l':
		case 'L':
		    strcat (pResult, "left");
		    break;
		case 'r':
		case 'R':
		    strcat (pResult, "right");
		    break;
		case 'c':
		case 'C':
		    strcat (pResult, "center");
		    break;
		default:
		    strcat (pResult, "left");
		    break;
		};
		// place it vertically in the middle
		strcat (pResult, " valign=middle>");

		// convert_html3 the predeclarations
		if (pCS->m_pPredecl)
		    pCS->m_pPredecl->convert_html3 ();

		// convert_html3 the contents
		p->convert_html3();

		// convert_html3 the postdeclarations
		if (pCS->m_pPostdecl)
		    pCS->m_pPostdecl->convert_html3 ();
	    }
	    c.goNext();
	    d.goNext();
	} while (c.valid() && d.valid());
    }
}

void
ColList::convert_12083 (void)
{
    if (!m_pSpecifiers)
	return;

    // convert separate cells
    strcat (pResult, "\n<arrayrow>");
    if (!empty() && !m_pSpecifiers->empty()) {
	ListCursor c(*this);
	ListCursor d(*m_pSpecifiers);
	c.goFirst();
	d.goFirst();
	do {
	    Node *p = (Node *) c;
	    ColSpecifier *pCS = (ColSpecifier *)d;
	    if (p) {
		strcat (pResult, "<arraycol><arraycel>");
		// convert_iso12038 the predeclarations
		if (pCS->m_pPredecl)
		    pCS->m_pPredecl->convert_12083 ();

		// convert_12083 the contents
		p->convert_12083();

		// convert_12083 the postdeclarations
		if (pCS->m_pPostdecl)
		    pCS->m_pPostdecl->convert_12083 ();
	    }
	    c.goNext();
	    d.goNext();
	} while (c.valid() && d.valid());
    }
}
