/***************************************************************************\
 *                                     Tab2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

#include "node.h"
#include "maths.h"
#include "rtti.h"

#ifndef INCL_LIST_H
#define INCL_LIST_H 1

class ListNode {
    friend class List;
    friend class ListCursor;
    Node *data;
    ListNode *previous, *next;
    
    ListNode(Node *d, ListNode *p = NULL, ListNode *n = NULL) : data(d) {
	previous = p;
	next = n;
    }
};


class List : public Node
{
    friend class ListCursor;
private:
    ListNode *m_pFirst, *m_pLast;
public:
    DECLARE_RTTI (List);

    List(void) { m_pFirst = m_pLast = NULL; }
    List(List &);
    ~List(void);

    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);


    void	assign(List &);

    Node	*getFirst(void) { return m_pFirst->data; }
    Node	*getLast(void) { return m_pLast->data; }

    void	addFirst(Node *);
    void	addLast(Node *);
    void	dropFirst(void);
    void	dropLast(void);

    Boolean	empty(void) { return BOOLEAN(m_pFirst == NULL); }
    Boolean	singleton(void) { return BOOLEAN(m_pFirst == m_pLast); }
    int		items(void);
};

class ColList : public List
{
public:
    ColList ();
    ~ColList ();

    virtual void	specifiers (List *pSpecifiers);
    virtual void	convert_html3 (void);
    virtual void	convert_12083 (void);

private:
    List	*m_pSpecifiers;
};

class ListCursor {
    friend class List;
private:
    List *list;
    ListNode *cursor;
public:
    ListCursor(List &l) { list = &l; cursor = list->m_pFirst; }
    void	assign(ListCursor &l) { list = l.list; cursor = l.cursor; }
    
    Boolean	valid(void) { return BOOLEAN(cursor != NULL); }

    void	goPrevious(void) { cursor = cursor->previous; }
    void	goNext(void) { cursor = cursor->next; }
    void	goFirst(void) { cursor = list->m_pFirst; }
    void	goLast(void) { cursor = list->m_pLast; }

    Boolean	atFirst(void) { return BOOLEAN(cursor->previous == NULL); }
    Boolean	atLast(void) { return BOOLEAN(cursor->next == NULL); }

    void	dropNext(void);
    void	dropPrevious(void);

    void	addNext(Node *);
    void	addPrevious(Node *);

    Node	*get(void) { return (Node *) cursor->data; }

    Node	*getPrevious(void) { return cursor->previous->data; }
    Node	*getNext(void) { return cursor->next->data; }
    Node	*getFirst(void) { return list->getFirst(); }
    Node	*getLast(void) { return list->getLast(); }

    operator	Node *() { return get(); }
    Node	*operator->() { return get(); }
};

#endif
