/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

static char *rcsid = "$Revision$";

#include <unistd.h>
#include "phase.h"

#define HTML3_MAPPING "/afs/cern.ch/user/j/jsaarela/math2html/math2html.cfg"
#define ISO12083_MAPPING "/afs/cern.ch/user/j/jsaarela/math2html/iso12083.cfg"

extern int yydebug;

main(int argc, char *argv[])
{
    // produce html3 by default
    int			html3 = 1;
    int			c;
    extern int		optind;
    extern char		*optarg;
    FILE		*sourceFile;
    int			errflg = 0;
    char		*pszFilename, *pszConfigfile;
    ConfigurationFile	*pUserConfig = NULL;

    while ((c = getopt(argc, argv, "idc:")) != EOF) {
	switch (c) {
	case 'i':
	    html3 = 0;
	    break;
	case 'd':
	    yydebug = 1;
	    break;
	case 'c':
	    pszConfigfile = optarg;
	    break;
	case '?':
	    errflg++;
	}
	
    }

    pszFilename = argv[optind];
    if (!pszFilename)
	errflg++;

    if (errflg) {
	cerr << "usage: " << argv[0]
	    << " [-d] [-c configuration file] [-i] sourcefile" << endl;
	exit(1);
    }

    if (!(sourceFile = fopen(pszFilename, "r"))) {
	cerr << argv[0] << ": cannot find sourcefile '"
	    << pszFilename << "'" << endl;
	exit(1);
    }

    // allocate piece of memory for the LaTeX source code
    // as well as for the result
    char *pSourceCode;
    pSource = pSourceCode = (char *)malloc (10240);
    pResult = (char *)malloc (10240);
    if (!pSourceCode || !pResult)
	exit (1);

    // open the system configuration file
    config = new ConfigurationFile;
    if (html3)
	config->open (HTML3_MAPPING);
    else
	config->open (ISO12083_MAPPING);

    // open the optional configuration file
    if (pszConfigfile) {
	pUserConfig = new ConfigurationFile;
	if (pUserConfig->open (pszConfigfile))
	    cerr << argv[0] << ": could not open configuration file "
		<< pszConfigfile << endl;
	else {
	    Pair *pPair;
	    while (pPair = pUserConfig->nextEntry()) {
		mappings.Add (pPair);
	    }
	}
    }

    // read in the whole file and put it into pSource
    int		count;
    char	buffer[256];

    *pSource = '\0';
    while (fgets (buffer, sizeof(buffer)-1, sourceFile))
	strcat (pSource, buffer);

    fclose (sourceFile);

    // read the configuration file
    phase1 ();
    
    // start parsing
    if (phase2 ()) {
	strcpy (pResult, "cannot parse");
    } else {
	// parsing succeeded, start the conversion
	if (html3)
	    convert_html3 ();
	else
	    convert_12083 ();
    }

    // print the result
    cout << pResult;

    if (pUserConfig)
	delete pUserConfig;
    free (pResult);
    free (pSourceCode);

    return 0;
}
