/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

#include <memory.h>
#include <stdlib.h>
#include "map.h"

StringMap::StringMap (int nSize)
{
    m_pHashTable = new Node*[nSize+1];
    memset (m_pHashTable, 0, nSize*sizeof(Node *));
    m_iSize = nSize;

}

StringMap::~StringMap ()
{
    Node *pNode, *pNextNode;

    for (int x = 0; x < m_iSize; x++) {
	pNode = m_pHashTable[x];
	while (pNode) {
	    pNextNode = pNode->pNextNode;
	    free (pNode->pPair->key);
	    free (pNode->pPair->value);
	    delete pNode->pPair;
	    delete pNode;
	    pNode = pNextNode;
	}
    }
    delete [] m_pHashTable;
}

void
StringMap::Add (Pair *pPair)
{
    ASSERT (pPair);
    ASSERT (pPair->key);

    unsigned	uHashValue = hash (pPair->key);
    Node	*pNode = m_pHashTable[uHashValue];
    Node	*pNewNode = new Node;

    m_pHashTable[uHashValue] = pNewNode;
    pNewNode->pPair = pPair;
    pNewNode->pNextNode = pNode;
}

// calculate the hash value
// this algorithm is based on the one of Chris Torek (torek@elf.ee.lbl.gov) 
// 33 is so-called magic number

unsigned
StringMap::hash (char *pszStr)
{
    ASSERT (pszStr);
    ASSERT (strlen(pszStr));

    unsigned h = 0;

    while (*pszStr)
	h = h * 33 + *pszStr++;
    return h % m_iSize;
}    

PSTR
StringMap::operator[] (PSTR pszKey)
{
    ASSERT (pszKey);
    ASSERT (strlen(pszKey));

    Node	*pNode = m_pHashTable[hash(pszKey)];

    while (pNode) {
	if (!strcmp (pNode->pPair->key, pszKey))
	    return pNode->pPair->value;
	pNode = pNode->pNextNode;
    }
    return NULL;
}
