/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

static char *rcsid = "$Revision$";

#include "maths.h"
#include "map.h"

extern StringMap mappings;

Sum::Sum (Node *pSubscript, Node *pSupscript)
{
    subscript (pSubscript);
    superscript (pSupscript);
}

Sum::~Sum ()
{}

void
Sum::convert_html3 (void)
{
    strcat (pResult, "&sum;");
    Node::convert_html3 ();
}

void
Sum::convert_12083 (void)
{
    strcat (pResult, "&sum;");
    Node::convert_12083 ();
}

Integral::Integral (Node *pSubscript, Node *pSupscript)
{
    subscript (pSubscript);
    superscript (pSupscript);
}

Integral::~Integral ()
{}

void
Integral::convert_html3 (void)
{
    strcat (pResult, "&int;");
    Node::convert_html3 ();
}

void
Integral::convert_12083 (void)
{
    strcat (pResult, "&int;");
    Node::convert_12083 ();
}

Fraction::Fraction (Node *pOver,
		    Node *pUnder,
		    Node *pSupscript,
		    Node *pSubscript)
{
    m_pOver = pOver;
    m_pUnder = pUnder;
    superscript (pSupscript);
    subscript (pSubscript);
}

Fraction::~Fraction ()
{
    delete m_pOver;
    delete m_pUnder;
}

void
Fraction::convert_html3 (void)
{
    strcat (pResult, "{"); // could be <box>
    if (m_pOver) {
	m_pOver->convert_html3 ();
    }
    strcat (pResult, "<over>");
    if (m_pUnder) {
	m_pUnder->convert_html3 ();
    }
    strcat (pResult, "}"); // could be </box>
    Node::convert_html3 ();
}

void
Fraction::convert_12083 (void)
{
    strcat (pResult, "<fraction>");
    if (m_pOver) {
	strcat (pResult, "<num>");
	m_pOver->convert_12083 ();
	strcat (pResult, "</num>");
    }
    if (m_pUnder) {
	strcat (pResult, "<den>");
	m_pUnder->convert_12083 ();
	strcat (pResult, "</den>");
    }
    strcat (pResult, "</fraction>");
    Node::convert_12083 ();
}

Sequence::Sequence (List *pOptargs, char *text,
		    Node *pSubscript, Node *pSupscript)
{
    m_pszText = strdup (text);
    m_pOptargs = pOptargs;
    subscript (pSubscript);
    superscript (pSupscript);

    m_pParameters = NULL;
}

Sequence::Sequence (List *pOptargs, char *text,
		    Node *pSubscript, Node *pSupscript, List *pParameters)
{
    m_pszText = strdup (text);
    m_pOptargs = pOptargs;
    subscript (pSubscript);
    superscript (pSupscript);

    m_pParameters = pParameters;
}

Sequence::~Sequence ()
{
    free (m_pszText);
    if (m_pParameters)
	delete m_pParameters;
}

void
Sequence::convert_html3 (void)
{
    char endTag[1024], *result, *p = NULL;

    endTag[0] = '\0';
    if (result = mappings[m_pszText]) {
	// check the <noparam> case
	if (!strncmp (result, "<noparam>", 9)) {
	    result += 9*sizeof(char);
	    if (strlen (result))
		strcat (pResult, result);
	    return;
	} else
	    // check the special text1%stext2 case
	    // place text1 into pResult and text2 into endTag which
	    // will be appended into pResult at the end of this routine
	    
	    if ((p = (char *)strchr (result, '%'))
		&& *(p+sizeof(char)) == 's') {
		strncat (pResult, result, (p-result)*sizeof(char));
		p += 2 * sizeof(char);
		strcpy (endTag, p);
	    } else {
		strcat (pResult, result);
	    }
    }
   
    if (m_pParameters)
	m_pParameters->convert_html3();
    strcat (pResult, endTag);
    Node::convert_html3 ();
}

void
Sequence::convert_12083 (void)
{
    char endTag[1024], *result, *p = NULL;

    endTag[0] = '\0';
    if (result = mappings[m_pszText]) {
	// check the <noparam> case
	if (!strncmp (result, "<noparam>", 9)) {
	    result += 9*sizeof(char);
	    if (strlen (result))
		strcat (pResult, result);
	    return;
	} else
	    // check the special text1%stext2 case
	    // place text1 into pResult and text2 into endTag which
	    // will be appended into pResult at the end of this routine
	    
	    if ((p = (char *)strchr (result, '%'))
		&& *(p+sizeof(char)) == 's') {
		strncat (pResult, result, (p-result)*sizeof(char));
		p += 2 * sizeof(char);
		strcpy (endTag, p);
	    } else {
		strcat (pResult, result);
	    }
    }
   
    if (m_pParameters)
	m_pParameters->convert_12083 ();
    strcat (pResult, endTag);
    Node::convert_12083 ();
}

Plain::Plain (char *text, Node *pSubscript, Node *pSupscript)
{
    m_pszText = strdup (text);
    subscript (pSubscript);
    superscript (pSupscript);
}

Plain::~Plain ()
{
    free (m_pszText);
}

void
Plain::convert_html3 (void)
{
    char *p = m_pszText;

    while (*p) {
	switch (*p) {
	case '<':
	    strcat (pResult, "&lt;");
	    break;
	case '>':
	    strcat (pResult, "&gt;");
	    break;
	case '&':
	    strcat (pResult, "&amp;");
	    break;
	default:
	    strncat (pResult, p, 1);
	}
	p++;
    }
    Node::convert_html3 ();
}

void
Plain::convert_12083 (void)
{
    char *p = m_pszText;

    while (*p) {
	switch (*p) {
	case '<':
	    strcat (pResult, "&lt;");
	    break;
	case '>':
	    strcat (pResult, "&gt;");
	    break;
	case '&':
	    strcat (pResult, "&amp;");
	    break;
	default:
	    strncat (pResult, p, 1);
	}
	p++;
    }
    Node::convert_12083 ();
}

char *
Plain::text (void)
{
    return m_pszText;
}

Group::Group (List *pNodes)
{
    m_pNodes = pNodes;
}

Group::~Group ()
{
    if (m_pNodes)
	delete m_pNodes;
}

void
Group::convert_html3 ()
{
    if (m_pNodes)
	m_pNodes->convert_html3();
    Node::convert_html3 ();
}

void
Group::convert_12083 ()
{
    if (m_pNodes)
	m_pNodes->convert_12083();
    Node::convert_12083 ();
}

Table::Table (List *pSpecifiers, List *pEntries)
{
    m_pSpecifiers = pSpecifiers;
    m_pEntries = pEntries;
}

Table::~Table ()
{
    if (m_pEntries)
	delete m_pEntries;
    if (m_pSpecifiers)
	delete m_pSpecifiers;
}

void
Table::convert_html3 (void)
{
    // determine whether we create borders to this table or not
    // heuristics: find a vertical element in the specifiers
    int foundBar = 0;

    ListCursor b(*m_pSpecifiers);
    b.goFirst();
    do {
	// all elements in this list will be ColSpecifiers
	ColSpecifier *p = (ColSpecifier *) b;
	if (p->hasVerticalBar()) {
	    foundBar = 1;
	    break;
	}
	b.goNext ();
    } while (b.valid());

    if (foundBar)
	strcat (pResult, " border>");
    else
	strcat (pResult, ">");

    if (m_pEntries) {
	if (!m_pEntries->empty()) {
	    ListCursor c(*m_pEntries);
	    c.goFirst();
	    do {
		// all items in c are of type ColList
		ColList *pList = (ColList *) c;
		if (pList) {
		    pList->specifiers (m_pSpecifiers);
		    pList->convert_html3();
		}
		c.goNext();
	    } while (c.valid());
	}
    }
}

void
Table::convert_12083 (void)
{
    // generate the NMTOKENS for colsep

    strcat (pResult, " colsep=\"");

    ListCursor b(*m_pSpecifiers);
    b.goFirst();
    do {
	// all elements in this list will be ColSpecifiers
	ColSpecifier *p = (ColSpecifier *) b;
	if (p->hasVerticalBar()) {
	    strcat (pResult, "| ");
	} else {
	    strcat (pResult, "&space; ");
	}
	b.goNext ();
    } while (b.valid());

    strcat (pResult, "\" ");

    // generate the NMTOKENS for colalign
    strcat (pResult, "colalign=\"");
    ListCursor c(*m_pSpecifiers);
    c.goFirst();
    do {
	ColSpecifier *pCS = (ColSpecifier *)c;
	if (pCS) {
	    switch (pCS->m_cAlignment) {
	    case 'l':
	    case 'L':
		strcat (pResult, "left ");
		break;
	    case 'r':
	    case 'R':
		strcat (pResult, "right ");
		break;
	    case 'c':
	    case 'C':
		strcat (pResult, "center ");
		break;
	    default:
		strcat (pResult, "left ");
		break;
	    };
	}
	c.goNext();
    } while (c.valid());
    strcat (pResult, "\">");

    if (m_pEntries) {
	if (!m_pEntries->empty()) {
	    ListCursor c(*m_pEntries);
	    c.goFirst();
	    do {
		// all items in c are of type ColList
		ColList *pList = (ColList *) c;
		if (pList) {
		    pList->specifiers (m_pSpecifiers);
		    pList->convert_12083();
		}
		c.goNext();
	    } while (c.valid());
	}
    }
}

EqnTable::EqnTable (List *pEntries) : Table (NULL, pEntries)
{
}

EqnTable::EqnTable (List *pSpecifiers, List *pEntries) : Table (pSpecifiers, pEntries)
{
}

EqnTable::~EqnTable ()
{
}

void
EqnTable::convert_html3 (void)
{
    // create temporarily a list of ColSpecifiers which specify the
    // looks of an eqnarray (eg. alignments are right, center, left)
    if (!m_pSpecifiers) {
	m_pSpecifiers = new List;
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'r', NULL, NULL, NULL));
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'c', NULL, NULL, NULL));
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'l', NULL, NULL, NULL));
    }
			
    if (m_pEntries) {
	if (!m_pEntries->empty()) {
	    ListCursor c(*m_pEntries);
	    c.goFirst();
	    do {
		// all items are of type ColList
		ColList *pList = (ColList *) c;
		if (pList) {
		    pList->specifiers (m_pSpecifiers);
		    pList->convert_html3();
		}
		c.goNext();
	    } while (c.valid());
	}
    }
}

void
EqnTable::convert_12083 (void)
{
    // create temporarily a list of ColSpecifiers which specify the
    // looks of an eqnarray (eg. alignments are right, center, left)
    if (!m_pSpecifiers) {
	m_pSpecifiers = new List;
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'r', NULL, NULL, NULL));
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'c', NULL, NULL, NULL));
	m_pSpecifiers->addLast (new ColSpecifier
				(NULL, 'l', NULL, NULL, NULL));
    }

    if (m_pEntries) {
	if (!m_pEntries->empty()) {
	    ListCursor c(*m_pEntries);
	    c.goFirst();
	    do {
		// all items are of type ColList
		ColList *pList = (ColList *) c;
		if (pList) {
		    pList->specifiers (m_pSpecifiers);
		    pList->convert_12083();
		}
		c.goNext();
	    } while (c.valid());
	}
    }
}

Root::Root (List *pDegree, Node *pContents)
{
    m_pDegree = pDegree;
    m_pContents = pContents;
}

Root::~Root ()
{
    if (m_pDegree)
	delete m_pDegree;
    if (m_pContents)
	delete m_pContents;
}

void
Root::convert_html3 (void)
{
    // create <sqrt></sqrt> tags for roots without degrees and
    // <root><of></root> tags for others
    if (m_pDegree) {
	strcat (pResult, "<root>");
	m_pDegree->convert_html3 ();
	strcat (pResult, "<of>");
    } else {
	strcat (pResult, "<sqrt>");
    }
    if (m_pContents)
	m_pContents->convert_html3 ();
    if (m_pDegree)
	strcat (pResult, "</root>");
    else
	strcat (pResult, "</sqrt>");
}

void
Root::convert_12083 (void)
{
    strcat (pResult, "<radical>");
    if (m_pDegree) {
	strcat (pResult, "<radix>");
	m_pDegree->convert_12083 ();
	strcat (pResult, "</radix>");
    }
    if (m_pContents)
	m_pContents->convert_12083 ();
    strcat (pResult, "</radical>");
}

WidthSpecifier::WidthSpecifier (char cType, List *pDecls)
{
    m_cType = cType;
    m_pDecls = pDecls;
}

WidthSpecifier::~WidthSpecifier ()
{
    delete m_pDecls;
}

ColSpecifier::ColSpecifier (List *pPredecl,
		  char cAlignment,
		  WidthSpecifier *pWidth,
		  List *pPostdecl,
		  Node *pMisc)
{
    m_pPredecl = pPredecl;
    m_pPostdecl = pPostdecl;
    m_pWidth = pWidth;
    m_pMisc = pMisc;

    // if the alignments has not been specified, use left as default
    if (cAlignment != '\0')
	m_cAlignment = cAlignment;
    else
	m_cAlignment = 'l';
}

ColSpecifier::~ColSpecifier ()
{
    delete m_pPredecl;
    delete m_pPostdecl;
    delete m_pWidth;
    delete m_pMisc;
}

int
ColSpecifier::hasVerticalBar (void)
{
    if (!m_pMisc)
	return 0;
    return m_pMisc->RTTIMatch ("BarElement");
}
