%{
/***************************************************************************\
 *                                     Math2HTML                            *
 *                                     ------------------------------------ *
 *                                     Copyright  1995 Janne Saarela/CERN  *
 \**************************************************************************/

/*
 * $Log$
 */

static char *rcsid = "$Revision$";

#include "node.h"
#include "maths.h"
#include "list.h"
    
#define YYERROR_VERBOSE 1

extern List parsed;

int yyerror(char *msg)
{
//  cerr << msg;
  return 0;
}

extern int yylex(void);
extern void yyrestart (FILE *);

%}

%start main

%union {
  char c;
  char *s;
  Node *node;
  List *list;
  WidthSpecifier *width;
}

%token <s> SEQUENCE PLAIN
%token <c> LEFT_ALIGN RIGHT_ALIGN CENTER_ALIGN
%token <c> M_WIDTH B_WIDTH P_WIDTH AT_WIDTH
%token BAR EXCLAMATION LEFT RIGHT
%token SUM FRACTION INTEGRAL ROOT
%token BEGIN_GROUP END_GROUP BEGIN_ARG END_ARG MATHEMATICS
%token BEGIN_SUBSCRIPT BEGIN_SUPERSCRIPT
%token BEGIN_PREDECL BEGIN_POSTDECL
%token BEGIN_ARRAY BEGIN_TABBING BEGIN_TABULAR BEGIN_TABULAR_STAR
%token END_ARRAY END_TABBING END_TABULAR END_TABULAR_STAR
%token BEGIN_LONGTABLE END_LONGTABLE
%token BEGIN_EQNARRAY END_EQNARRAY
%token COLUMN_SEPARATOR END_OF_ROW

%type <list> main maths nested_maths row_entries columns col_defs defs
%type <list> eqn_row_entries eqn_columns pre_decl post_decl
%type <node> symbols nested_symbols subscript superscript table eqn_table misc
%type <c> alignment
%type <width> parametrised_decls

%%

main: main MATHEMATICS maths MATHEMATICS
				{ parsed.addLast (new BeginMaths);
				  parsed.addLast ($3);
				  parsed.addLast (new EndMaths); }
	| main table		{ parsed.addLast (new BeginTable);
				  parsed.addLast ($2);
				  parsed.addLast (new EndTable); }
	| main eqn_table	{ parsed.addLast (new BeginEqnTable);
				  parsed.addLast ($2);
				  parsed.addLast (new EndTable); }
	| main MATHEMATICS BEGIN_ARRAY col_defs eqn_row_entries END_ARRAY MATHEMATICS
				{ parsed.addLast (new BeginEqnTable);
				  EqnTable *pTable = new EqnTable($4, $5);
				  parsed.addLast (pTable);
				  parsed.addLast (new EndTable); }
	|			{ ; }
	;

maths: maths symbols			{ $1->addLast ($2); $$ = $1; }
	| symbols			{ $$ = new List;
					  $$->addLast ($1); }
	;

nested_maths: nested_maths MATHEMATICS maths MATHEMATICS
					{ $1->addLast (new BeginMaths);
					  $1->addLast ($3);
 				          $1->addLast (new EndMaths);
					  $$ = $1; }
	| MATHEMATICS maths MATHEMATICS	{ $$ = new List;
       					  $$->addFirst (new EndMaths);
					  $$->addFirst ($2);
				          $$->addFirst (new BeginMaths); }
	| nested_maths maths		{ $1->addLast ($2); $$ = $1; }
	| maths				{ $$ = new List;
    					  $$->addFirst ($1); }
	;

nested_symbols: BEGIN_GROUP maths END_GROUP
					{ $$ = new Group ($2); }
	;

symbols: BEGIN_ARRAY col_defs eqn_row_entries END_ARRAY
				{ List *pList = new List;
				  EqnTable *pTable = new EqnTable($2, $3);
				  pList->addLast (new EndMaths);
				  pList->addLast (new BeginEqnTable);
				  pList->addLast (pTable);
				  pList->addLast (new EndTable);
				  pList->addLast (new BeginMaths);
				  $$ = pList; }
	| LEFT				{ $$ = new Left; }
	| RIGHT				{ $$ = new Right; }
	| SUM 				{ $$ = new Sum (NULL, NULL); }
	| SUM subscript 		{ $$ = new Sum ($2, NULL); }
	| SUM superscript		{ $$ = new Sum (NULL, $2); }
	| SUM superscript subscript	{ $$ = new Sum ($3, $2); }
	| SUM subscript superscript	{ $$ = new Sum ($2, $3); }
	| PLAIN				{ $$ = new Plain ($1, NULL, NULL); }
	| PLAIN subscript		{ $$ = new Plain ($1, $2, NULL); }
	| PLAIN superscript		{ $$ = new Plain ($1, NULL, $2); }
	| PLAIN superscript subscript	{ $$ = new Plain ($1, $3, $2); }
	| PLAIN subscript superscript	{ $$ = new Plain ($1, $2, $3); }

	| ROOT BEGIN_ARG maths END_ARG nested_symbols
					{ $$ = new Root ($3, $5); }
	| ROOT nested_symbols
					{ $$ = new Root (NULL, $2); }
	| INTEGRAL			{ $$ = new Integral (NULL, NULL); }
	| INTEGRAL subscript		{ $$ = new Integral ($2, NULL); }
	| INTEGRAL superscript		{ $$ = new Integral (NULL, $2); }
	| INTEGRAL subscript superscript{ $$ = new Integral ($2, $3); }
	| INTEGRAL superscript subscript{ $$ = new Integral ($3, $2); }
	| FRACTION nested_symbols nested_symbols
   			{ $$ = new Fraction ($2, $3, NULL, NULL); }
	| FRACTION nested_symbols nested_symbols subscript
   			{ $$ = new Fraction ($2, $3, $4, NULL); }
	| FRACTION nested_symbols nested_symbols superscript
   			{ $$ = new Fraction ($2, $3, NULL, $4); }
	| FRACTION nested_symbols nested_symbols subscript superscript
   			{ $$ = new Fraction ($2, $3, $4, $5); }
	| FRACTION nested_symbols nested_symbols superscript subscript
   			{ $$ = new Fraction ($2, $3, $5, $4); }

	| SEQUENCE BEGIN_ARG maths END_ARG BEGIN_GROUP maths END_GROUP
			{ $$ = new Sequence ($3, $1, NULL, NULL, $6); }
	| SEQUENCE BEGIN_ARG maths END_ARG BEGIN_GROUP END_GROUP
			{ $$ = new Sequence ($3, $1, NULL, NULL, NULL); }
	| SEQUENCE BEGIN_ARG maths END_ARG BEGIN_ARG maths END_ARG BEGIN_GROUP maths END_GROUP
			{ $$ = new Sequence ($3, $1, NULL, NULL, $9); }

	| SEQUENCE BEGIN_GROUP END_GROUP
			{ $$ = new Sequence (NULL, $1, NULL, NULL, NULL); }
	| SEQUENCE BEGIN_GROUP END_GROUP subscript
			{ $$ = new Sequence (NULL, $1, $4, NULL, NULL); }
	| SEQUENCE BEGIN_GROUP END_GROUP superscript 
			{ $$ = new Sequence (NULL, $1, NULL, $4, NULL); }
	| SEQUENCE BEGIN_GROUP END_GROUP superscript subscript
			{ $$ = new Sequence (NULL, $1, $5, $4, NULL); }
	| SEQUENCE BEGIN_GROUP END_GROUP subscript superscript
			{ $$ = new Sequence (NULL, $1, $4, $5, NULL); }

	| SEQUENCE BEGIN_GROUP maths END_GROUP
			{ $$ = new Sequence (NULL, $1, NULL, NULL, $3); }
	| SEQUENCE BEGIN_GROUP maths END_GROUP subscript
			{ $$ = new Sequence (NULL, $1, $5, NULL, $3); }
	| SEQUENCE BEGIN_GROUP maths END_GROUP superscript 
			{ $$ = new Sequence (NULL, $1, NULL, $5, $3); }
	| SEQUENCE BEGIN_GROUP maths END_GROUP superscript subscript
			{ $$ = new Sequence (NULL, $1, $6, $5, $3); }
	| SEQUENCE BEGIN_GROUP maths END_GROUP subscript superscript
			{ $$ = new Sequence (NULL, $1, $5, $6, $3); }

	| SEQUENCE subscript superscript
			{ $$ = new Sequence (NULL, $1, $2, $3); }
	| SEQUENCE superscript subscript
			{ $$ = new Sequence (NULL, $1, $3, $2); }
	| SEQUENCE subscript
			{ $$ = new Sequence (NULL, $1, $2, NULL); }
	| SEQUENCE superscript
			{ $$ = new Sequence (NULL, $1, NULL, $2); }
	| SEQUENCE
			{ $$ = new Sequence (NULL, $1, NULL, NULL); }
	| BEGIN_GROUP END_GROUP	{ $$ = new List; }
	| BEGIN_GROUP nested_maths END_GROUP	{ $$ = new Group ($2); }
	| BEGIN_GROUP nested_maths END_GROUP subscript
					{ $$ = new Group ($2);
					  $$->subscript ($4); }
	| BEGIN_GROUP nested_maths END_GROUP superscript
					{ $$ = new Group ($2);
					  $$->superscript ($4); }
	| BEGIN_GROUP nested_maths END_GROUP subscript superscript
					{ $$ = new Group ($2);
					  $$->subscript ($4);
					  $$->superscript ($5); }
	| BEGIN_GROUP nested_maths END_GROUP superscript subscript
					{ $$ = new Group ($2);
					  $$->subscript ($5);
					  $$->superscript ($4); }
	;

subscript: BEGIN_SUBSCRIPT nested_symbols
					{ $$ = $2; }
	;

superscript: BEGIN_SUPERSCRIPT nested_symbols
					{ $$ = $2; }
	;

col_defs: BEGIN_GROUP defs END_GROUP	{ $$ = $2; }
	;

defs: defs pre_decl misc alignment post_decl
			{ $1->addLast (new ColSpecifier ($2, $4, NULL, $5, $3));
			  $$ = $1; }
	| defs pre_decl alignment post_decl
			{ $1->addLast (new ColSpecifier ($2, $3, NULL, $4, NULL));
			  $$ = $1; }
	| defs pre_decl alignment misc post_decl
			{ $1->addLast (new ColSpecifier ($2, $3, NULL, $5, $4));
			  $$ = $1; }
	| defs pre_decl alignment post_decl misc
			{ $1->addLast (new ColSpecifier ($2, $3, NULL, $4, $5));
			  $$ = $1; }
	| defs pre_decl misc parametrised_decls post_decl
			{ $1->addLast (new ColSpecifier ($2, '\0', $4, $5, $3));
			  $$ = $1; }
	| defs pre_decl parametrised_decls post_decl
			{ $1->addLast (new ColSpecifier ($2, '\0', $3, $4, NULL));
			  $$ = $1; }
	| defs pre_decl parametrised_decls misc post_decl
			{ $1->addLast (new ColSpecifier ($2, '\0', $3, $5, $4));
			  $$ = $1; }
	| defs pre_decl parametrised_decls post_decl misc
			{ $1->addLast (new ColSpecifier ($2, '\0', $3, $4, $5));
			  $$ = $1; }
	| BAR				{ $$ = new List;
    					  $$->addFirst (new ColSpecifier (NULL, '\0', NULL, NULL, new BarElement)); }
	|				{ $$ = new List; }
	;

alignment: LEFT_ALIGN			{ $$ = $1; }
	| RIGHT_ALIGN			{ $$ = $1; }
	| CENTER_ALIGN			{ $$ = $1; }
	;

misc: BAR				{ $$ = new BarElement; }
	| EXCLAMATION BEGIN_GROUP maths END_GROUP
					{ $$ = NULL; }
	| AT_WIDTH BEGIN_GROUP END_GROUP
					{ $$ = NULL; } 
	| AT_WIDTH BEGIN_GROUP nested_maths END_GROUP
					{ $$ = NULL; }
	;

pre_decl: BEGIN_PREDECL BEGIN_GROUP maths END_GROUP
					{ $$ = $3; }
	| 				{ $$ = NULL; }
	;

post_decl: BEGIN_POSTDECL BEGIN_GROUP maths END_GROUP
					{ $$ = $3; }
	| 				{ $$ = NULL; }
	;

parametrised_decls: M_WIDTH BEGIN_GROUP END_GROUP
					{ $$ = new WidthSpecifier ($1, NULL); }
	| P_WIDTH BEGIN_GROUP END_GROUP
					{ $$ = new WidthSpecifier ($1, NULL); }
	| B_WIDTH BEGIN_GROUP END_GROUP
					{ $$ = new WidthSpecifier ($1, NULL); }
	| M_WIDTH BEGIN_GROUP maths END_GROUP
					{ $$ = new WidthSpecifier ($1, $3); }
	| P_WIDTH BEGIN_GROUP maths END_GROUP
					{ $$ = new WidthSpecifier ($1, $3); }
	| B_WIDTH BEGIN_GROUP maths END_GROUP
					{ $$ = new WidthSpecifier ($1, $3); }
	;

position_defs: BEGIN_ARG maths END_ARG
	
table: BEGIN_ARRAY col_defs row_entries END_ARRAY
					{ $$ = new Table ($2, $3); }
	| BEGIN_ARRAY position_defs col_defs row_entries END_ARRAY
					{ $$ = new Table ($3, $4); }
	| BEGIN_TABULAR col_defs row_entries END_TABULAR
					{ $$ = new Table ($2, $3); }
	| BEGIN_TABULAR position_defs col_defs row_entries END_TABULAR
					{ $$ = new Table ($3, $4); }
	| BEGIN_TABBING col_defs row_entries END_TABBING
					{ $$ = new Table ($2, $3); }
	| BEGIN_TABULAR_STAR col_defs row_entries END_TABULAR_STAR
					{ $$ = new Table ($2, $3); }
	| BEGIN_LONGTABLE col_defs row_entries END_LONGTABLE
					{ $$ = new Table ($2, $3); }
	;

row_entries: row_entries END_OF_ROW columns
					{ $1->addLast ($3); $$ = $1; }
	| columns			{ $$ = new List; $$->addFirst ($1); }
	| row_entries END_OF_ROW	{ $$ = $1; }
	;

columns: columns COLUMN_SEPARATOR	{ $1->addLast (new List); $$ = $1; }
	| columns COLUMN_SEPARATOR nested_maths
					{ $1->addLast ($3); $$ = $1; }
	| columns nested_maths		{ $1->addLast ($2); $$ = $1; }
	| COLUMN_SEPARATOR		{ $$ = new ColList;
				          $$->addLast (new List) }
	| COLUMN_SEPARATOR COLUMN_SEPARATOR	{ $$ = new ColList;
						  $$->addLast (new List);
	  				          $$->addLast (new List); }
	| nested_maths			{ $$ = new ColList;
    					  $$->addFirst ($1); }
	;

eqn_table: BEGIN_EQNARRAY eqn_row_entries END_EQNARRAY
					{ $$ = new EqnTable ($2); }
	;
	
eqn_row_entries: eqn_row_entries END_OF_ROW eqn_columns
					{ $1->addLast ($3); $$ = $1; }
	| eqn_columns			{ $$ = new List; $$->addFirst ($1); }
	| eqn_row_entries END_OF_ROW	{ $$ = $1; }
	;

eqn_columns: COLUMN_SEPARATOR eqn_columns
					{ $2->addFirst (new List);
    					  $$ = $2; }
	| eqn_columns COLUMN_SEPARATOR	{ $1->addLast (new List); $$ = $1; }
	| eqn_columns COLUMN_SEPARATOR maths
					{ $3->addFirst (new BeginMaths);
					  $3->addLast (new EndMaths);
					  $1->addLast ($3);
				          $$ = $1; }
	| maths				{ $$ = new ColList;
					  $1->addLast (new EndMaths);
					  $1->addFirst (new BeginMaths);
					  $$->addFirst ($1); }
	;

%%
/*
*/
extern FILE *yyin;

int parse_file(void)
{
    yyin = NULL;
    // uncomment the following line in order to see how bison
    // parses the input
//  yydebug = 1; 

    int retval = yyparse();
    yyrestart (NULL);
    return retval;
}
